@@ -1,5 +1,7 @@
 {-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE MagicHash #-}
+{-# LANGUAGE TemplateHaskellQuotes #-}
 
 {- |
 Module      :  Generics.Deriving.TH.Internal
@@ -26,17 +28,21 @@ import           Data.Maybe (mapMaybe)
 import qualified Data.Set as Set
 import           Data.Set (Set)
 
-import           Language.Haskell.TH.Datatype
+import qualified Generics.Deriving as GD
+import           Generics.Deriving hiding
+                   ( DecidedStrictness(..), Fixity(Infix)
+                   , SourceStrictness(..), SourceUnpackedness(..)
+                   , datatypeName
+                   )
+
+import           GHC.Exts (Addr#, Char#, Double#, Float#, Int#, Word#)
+
+import           Language.Haskell.TH.Datatype as Datatype
 import           Language.Haskell.TH.Datatype.TyVarBndr
 import           Language.Haskell.TH.Lib
 import           Language.Haskell.TH.Ppr (pprint)
 import           Language.Haskell.TH.Syntax
 
-#ifndef CURRENT_PACKAGE_KEY
-import           Data.Version (showVersion)
-import           Paths_generic_deriving (version)
-#endif
-
 -------------------------------------------------------------------------------
 -- Expanding type synonyms
 -------------------------------------------------------------------------------
@@ -44,11 +50,7 @@ import           Paths_generic_deriving (version)
 type TypeSubst = Map Name Type
 
 applySubstitutionKind :: Map Name Kind -> Type -> Type
-#if MIN_VERSION_template_haskell(2,8,0)
 applySubstitutionKind = applySubstitution
-#else
-applySubstitutionKind _ t = t
-#endif
 
 substNameWithKind :: Name -> Kind -> Type -> Type
 substNameWithKind n k = applySubstitutionKind (Map.singleton n k)
@@ -60,10 +62,15 @@ substNamesWithKindStar ns t = foldr' (flip substNameWithKind starK) t ns
 -- StarKindStatus
 -------------------------------------------------------------------------------
 
--- | Whether a type is not of kind *, is of kind *, or is a kind variable.
-data StarKindStatus = NotKindStar
-                    | KindStar
+-- | Whether a type is of kind @*@, a kind variable, or some other kind. The
+-- kind variable case is given special treatment solely to support GHC 8.0 and
+-- earlier, in which Generic1 was not poly-kinded. In order to support deriving
+-- Generic1 instances on these versions of GHC, we must substitute such kinds
+-- with @*@ to ensure that the resulting instance is well kinded.
+-- See @Note [Generic1 is polykinded in base-4.10]@ in "Generics.Deriving.TH".
+data StarKindStatus = KindStar
                     | IsKindVar Name
+                    | OtherKind
   deriving Eq
 
 -- | Does a Type have kind * or k (for some kind variable k)?
@@ -71,10 +78,8 @@ canRealizeKindStar :: Type -> StarKindStatus
 canRealizeKindStar t
   | hasKindStar t = KindStar
   | otherwise = case t of
-#if MIN_VERSION_template_haskell(2,8,0)
                      SigT _ (VarT k) -> IsKindVar k
-#endif
-                     _               -> NotKindStar
+                     _               -> OtherKind
 
 -- | Returns 'Just' the kind variable 'Name' of a 'StarKindStatus' if it exists.
 -- Otherwise, returns 'Nothing'.
@@ -94,11 +99,7 @@ catKindVarNames = mapMaybe starKindStatusToName
 -- | Returns True if a Type has kind *.
 hasKindStar :: Type -> Bool
 hasKindStar VarT{}         = True
-#if MIN_VERSION_template_haskell(2,8,0)
 hasKindStar (SigT _ StarT) = True
-#else
-hasKindStar (SigT _ StarK) = True
-#endif
 hasKindStar _              = False
 
 -- | Converts a VarT or a SigT into Just the corresponding TyVarBndr.
@@ -139,58 +140,46 @@ makeFunType argTys resTy = foldr' (AppT . AppT ArrowT) resTy argTys
 -- k1 -> k2 -> k3
 -- @
 makeFunKind :: [Kind] -> Kind -> Kind
-#if MIN_VERSION_template_haskell(2,8,0)
 makeFunKind = makeFunType
-#else
-makeFunKind argKinds resKind = foldr' ArrowK resKind argKinds
-#endif
 
--- | Detect if a Name occurs as an argument to some type family. This makes an
--- effort to exclude /oversaturated/ arguments to type families. For instance,
--- if one declared the following type family:
---
--- @
--- type family F a :: Type -> Type
--- @
---
--- Then in the type @F a b@, we would consider @a@ to be an argument to @F@,
--- but not @b@.
-isInTypeFamilyApp :: Name -> Type -> [Type] -> Q Bool
-isInTypeFamilyApp name tyFun tyArgs =
-  case tyFun of
-    ConT tcName -> go tcName
-    _           -> return False
+-- | Remove any outer `SigT` and `ParensT` constructors, and turn
+-- an outermost `InfixT` constructor into plain applications.
+dustOff :: Type -> Type
+dustOff (SigT ty _) = dustOff ty
+dustOff (ParensT ty) = dustOff ty
+dustOff (InfixT ty1 n ty2) = ConT n `AppT` ty1 `AppT` ty2
+dustOff ty = ty
+
+-- | Checks whether a type is an unsaturated type family
+-- application.
+isUnsaturatedType :: Type -> Q Bool
+isUnsaturatedType = go 0 . dustOff
   where
-    go :: Name -> Q Bool
-    go tcName = do
+    -- Expects its argument to be dusted
+    go :: Int -> Type -> Q Bool
+    go d t = case t of
+      ConT tcName -> check d tcName
+      AppT f _ -> go (d + 1) (dustOff f)
+      _ -> return False
+
+    check :: Int -> Name -> Q Bool
+    check d tcName = do
+      mbinders <- getTypeFamilyBinders tcName
+      return $ case mbinders of
+        Just bndrs -> length bndrs > d
+        Nothing -> False
+
+-- | Given a name, check if that name is a type family. If
+-- so, return a list of its binders.
+getTypeFamilyBinders :: Name -> Q (Maybe [TyVarBndrVis])
+getTypeFamilyBinders tcName = do
       info <- reify tcName
-      case info of
-#if MIN_VERSION_template_haskell(2,11,0)
+      return $ case info of
         FamilyI (OpenTypeFamilyD (TypeFamilyHead _ bndrs _ _)) _
-          -> withinFirstArgs bndrs
-#elif MIN_VERSION_template_haskell(2,7,0)
-        FamilyI (FamilyD TypeFam _ bndrs _) _
-          -> withinFirstArgs bndrs
-#else
-        TyConI (FamilyD TypeFam _ bndrs _)
-          -> withinFirstArgs bndrs
-#endif
-
-#if MIN_VERSION_template_haskell(2,11,0)
+          -> Just bndrs
         FamilyI (ClosedTypeFamilyD (TypeFamilyHead _ bndrs _ _) _) _
-          -> withinFirstArgs bndrs
-#elif MIN_VERSION_template_haskell(2,9,0)
-        FamilyI (ClosedTypeFamilyD _ bndrs _ _) _
-          -> withinFirstArgs bndrs
-#endif
-
-        _ -> return False
-      where
-        withinFirstArgs :: [a] -> Q Bool
-        withinFirstArgs bndrs =
-          let firstArgs = take (length bndrs) tyArgs
-              argFVs    = freeVariables firstArgs
-          in return $ name `elem` argFVs
+          -> Just bndrs
+        _ -> Nothing
 
 -- | True if the type does not mention the Name
 ground :: Type -> Name -> Bool
@@ -204,29 +193,6 @@ applyTyToTys = List.foldl' AppT
 applyTyToTvbs :: Name -> [TyVarBndr_ flag] -> Type
 applyTyToTvbs = List.foldl' (\a -> AppT a . tyVarBndrToType) . ConT
 
--- | Split an applied type into its individual components. For example, this:
---
--- @
--- Either Int Char
--- @
---
--- would split to this:
---
--- @
--- [Either, Int, Char]
--- @
-unapplyTy :: Type -> (Type, [Type])
-unapplyTy ty = go ty ty []
-  where
-    go :: Type -> Type -> [Type] -> (Type, [Type])
-    go _      (AppT ty1 ty2)     args = go ty1 ty1 (ty2:args)
-    go origTy (SigT ty' _)       args = go origTy ty' args
-#if MIN_VERSION_template_haskell(2,11,0)
-    go origTy (InfixT ty1 n ty2) args = go origTy (ConT n `AppT` ty1 `AppT` ty2) args
-    go origTy (ParensT ty')      args = go origTy ty' args
-#endif
-    go origTy _                  args = (origTy, args)
-
 -- | Split a type signature by the arrows on its spine. For example, this:
 --
 -- @
@@ -250,14 +216,7 @@ uncurryTy t = ([], [t])
 
 -- | Like uncurryType, except on a kind level.
 uncurryKind :: Kind -> ([TyVarBndrSpec], [Kind])
-#if MIN_VERSION_template_haskell(2,8,0)
 uncurryKind = uncurryTy
-#else
-uncurryKind (ArrowK k1 k2) =
-  let (kvbs, ks) = uncurryKind k2
-  in (kvbs, k1:ks)
-uncurryKind k = ([], [k])
-#endif
 
 tyVarBndrToType :: TyVarBndr_ flag -> Type
 tyVarBndrToType = elimTV VarT (\n k -> SigT (VarT n) k)
@@ -298,11 +257,7 @@ isTyVar _          = False
 
 -- | Is the given kind a variable?
 isKindVar :: Kind -> Bool
-#if MIN_VERSION_template_haskell(2,8,0)
 isKindVar = isTyVar
-#else
-isKindVar _ = False -- There are no kind variables
-#endif
 
 -- | Returns 'True' is a 'Type' contains no type variables.
 isTypeMonomorphic :: Type -> Bool
@@ -310,10 +265,7 @@ isTypeMonomorphic = go
   where
     go :: Type -> Bool
     go (AppT t1 t2) = go t1 && go t2
-    go (SigT t _k)  = go t
-#if MIN_VERSION_template_haskell(2,8,0)
-                           && go _k
-#endif
+    go (SigT t k)   = go t && go k
     go VarT{}       = False
     go _            = True
 
@@ -332,10 +284,7 @@ mentionsName = go
   where
     go :: Type -> [Name] -> Bool
     go (AppT t1 t2) names = go t1 names || go t2 names
-    go (SigT t _k)  names = go t names
-#if MIN_VERSION_template_haskell(2,8,0)
-                              || go _k names
-#endif
+    go (SigT t k)   names = go t names || go k names
     go (VarT n)     names = n `elem` names
     go _            _     = False
 
@@ -385,22 +334,40 @@ isNewtypeVariant (NewtypeInstance_ {}) = True
 -- | Indicates whether Generic or Generic1 is being derived.
 data GenericClass = Generic | Generic1 deriving Enum
 
--- | Like 'GenericArity', but bundling two things in the 'Gen1' case:
---
--- 1. The 'Name' of the last type parameter.
--- 2. If that last type parameter had kind k (where k is some kind variable),
---    then it has 'Just' the kind variable 'Name'. Otherwise, it has 'Nothing'.
-data GenericKind = Gen0
-                 | Gen1 Name (Maybe Name)
+-- | Records information about the type variables of a data type with a
+-- 'Generic' or 'Generic1' instance.
+data GenericTvbs
+    -- | Information about a data type with a 'Generic' instance.
+  = Gen0
+      { gen0Tvbs :: [TyVarBndrUnit]
+        -- ^ All of the type variable arguments to the data type.
+      }
+    -- | Information about a data type with a 'Generic1' instance.
+  | Gen1
+      { gen1InitTvbs :: [TyVarBndrUnit]
+        -- ^ All of the type variable arguments to the data type except the
+        --   last one. In a @'Generic1' (T a_1 ... a_(n-1))@ instance, the
+        --   'gen1InitTvbs' would be @[a_1, ..., a_(n-1)]@.
+      , gen1LastTvbName :: Name
+        -- ^ The name of the last type variable argument to the data type.
+        --   In a @'Generic1' (T a_1 ... a_(n-1))@ instance, the
+        --   'gen1LastTvbName' name would be @a_n@.
+     , gen1LastTvbKindVar :: Maybe Name
+        -- ^ If the 'gen1LastTvbName' has kind @k@, where @k@ is some kind
+        --   variable, then the 'gen1LastTvbKindVar' is @'Just' k@. Otherwise,
+        --   the 'gen1LastTvbKindVar' is 'Nothing'.
+      }
 
--- Determines the universally quantified type variables (possibly after
--- substituting * in the case of Generic1) and the last type parameter name
--- (if there is one).
-genericKind :: GenericClass -> [Type] -> ([TyVarBndrUnit], GenericKind)
-genericKind gClass tySynVars =
+-- | Compute 'GenericTvbs' from a 'GenericClass' and the type variable
+-- arguments to a data type.
+mkGenericTvbs :: GenericClass -> [Type] -> GenericTvbs
+mkGenericTvbs gClass tySynVars =
   case gClass of
-    Generic  -> (freeVariablesWellScoped tySynVars, Gen0)
-    Generic1 -> (freeVariablesWellScoped initArgs, Gen1 (varTToName lastArg) mbLastArgKindName)
+    Generic  -> Gen0{gen0Tvbs = freeVariablesWellScoped tySynVars}
+    Generic1 -> Gen1{ gen1InitTvbs       = freeVariablesWellScoped initArgs
+                    , gen1LastTvbName    = varTToName lastArg
+                    , gen1LastTvbKindVar = mbLastArgKindName
+                    }
   where
     -- Everything below is only used for Generic1.
     initArgs :: [Type]
@@ -413,6 +380,14 @@ genericKind gClass tySynVars =
     mbLastArgKindName = starKindStatusToName
                       $ canRealizeKindStar lastArg
 
+-- | Return the type variable arguments to a data type that appear in a
+-- 'Generic' or 'Generic1' instance. For a 'Generic' instance, this consists of
+-- all the type variable arguments. For a 'Generic1' instance, this consists of
+-- all the type variable arguments except for the last one.
+genericInitTvbs :: GenericTvbs -> [TyVarBndrUnit]
+genericInitTvbs (Gen0{gen0Tvbs = tvbs})     = tvbs
+genericInitTvbs (Gen1{gen1InitTvbs = tvbs}) = tvbs
+
 -- | A version of 'DatatypeVariant' in which the data family instance
 -- constructors come equipped with the 'ConstructorInfo' of the first
 -- constructor in the family instance (for 'Name' generation purposes).
@@ -469,7 +444,12 @@ derivingKindError tyConName = fail
     ( showString (nameBase tyConName)
     . showString " ..."
     )
-  . showString "‘\n\tClass Generic1 expects an argument of kind * -> *"
+  . showString "‘\n\tClass Generic1 expects an argument of kind "
+#if MIN_VERSION_base(4,10,0)
+  . showString "k -> *"
+#else
+  . showString "* -> *"
+#endif
   $ ""
 
 -- | The data type mentions the last type variable in a place other
@@ -480,10 +460,27 @@ outOfPlaceTyVarError = fail
   . showString " the last argument of a data type"
   $ ""
 
+-- | The data type mentions the last type variable in a type family
+-- application.
+typeFamilyApplicationError :: Q a
+typeFamilyApplicationError = fail
+  . showString "Constructor must not apply its last type variable"
+  . showString " to an unsaturated type family"
+  $ ""
+
+-- | We cannot define implementations for @from(1)@ or @to(1)@ at the term level
+-- for @type data@ declarations, which only exist at the type level.
+typeDataError :: Name -> Q a
+typeDataError dataName = fail
+  . showString "Cannot derive instance for ‘"
+  . showString (nameBase dataName)
+  . showString "‘, which is a ‘type data‘ declaration"
+  $ ""
+
 -- | Cannot have a constructor argument of form (forall a1 ... an. <type>)
 -- when deriving Generic(1)
-rankNError :: a
-rankNError = error "Cannot have polymorphic arguments"
+rankNError :: Q a
+rankNError = fail "Cannot have polymorphic arguments"
 
 -- | Boilerplate for top level splices.
 --
@@ -504,19 +501,29 @@ reifyDataInfo name = do
                   , datatypeVariant   = variant
                   , datatypeCons      = cons
                   } <- reifyDatatype name
-     let variant_ = case variant of
-                      Datatype        -> Datatype_
-                      Newtype         -> Newtype_
-                      -- This isn't total, but the API requires that the data
-                      -- family instance have at least one constructor anyways,
-                      -- so this will always succeed.
-                      DataInstance    -> DataInstance_    $ head cons
-                      NewtypeInstance -> NewtypeInstance_ $ head cons
+     variant_ <-
+       case variant of
+         Datatype          -> return Datatype_
+         Newtype           -> return Newtype_
+         DataInstance      -> return $ DataInstance_    $ headDataFamInstCon parentName cons
+         NewtypeInstance   -> return $ NewtypeInstance_ $ headDataFamInstCon parentName cons
+#if MIN_VERSION_th_abstraction(0,5,0)
+         Datatype.TypeData -> typeDataError parentName
+#endif
      checkDataContext parentName ctxt $ Right (parentName, tys, cons, variant_)
   where
     ns :: String
     ns = "Generics.Deriving.TH.reifyDataInfo: "
 
+    -- This isn't total, but the API requires that the data family instance have
+    -- at least one constructor anyways, so this will always succeed.
+    headDataFamInstCon :: Name -> [ConstructorInfo] -> ConstructorInfo
+    headDataFamInstCon dataFamName cons =
+      case cons of
+        con:_ -> con
+        [] -> error $ "reified data family instance without a data constructor: "
+                   ++ nameBase dataFamName
+
 -- | One cannot derive Generic(1) instance for anything that uses DatatypeContexts,
 -- so check to make sure the Cxt field of a datatype is null.
 checkDataContext :: Name -> Cxt -> a -> Q a
@@ -526,384 +533,298 @@ checkDataContext dataName _  _ = fail $
 
 -- | Deriving Generic(1) doesn't work with ExistentialQuantification or GADTs.
 checkExistentialContext :: Name -> [TyVarBndrUnit] -> Cxt -> Q ()
-checkExistentialContext conName vars ctxt =
+checkExistentialContext constrName vars ctxt =
   unless (null vars && null ctxt) $ fail $
-    nameBase conName ++ " must be a vanilla data constructor"
+    nameBase constrName ++ " must be a vanilla data constructor"
+
+#if !(MIN_VERSION_template_haskell(2,21,0)) && !(MIN_VERSION_th_abstraction(0,6,0))
+type TyVarBndrVis = TyVarBndrUnit
+
+bndrReq :: ()
+bndrReq = ()
+#endif
 
 -------------------------------------------------------------------------------
--- Manually quoted names
+-- Quoted names
 -------------------------------------------------------------------------------
 
--- By manually generating these names we avoid needing to use the
--- TemplateHaskell language extension when compiling the generic-deriving library.
--- This allows the library to be used in stage1 cross-compilers.
-
-gdPackageKey :: String
-#ifdef CURRENT_PACKAGE_KEY
-gdPackageKey = CURRENT_PACKAGE_KEY
-#else
-gdPackageKey = "generic-deriving-" ++ showVersion version
-#endif
-
-mkGD4'4_d :: String -> Name
-#if MIN_VERSION_base(4,6,0)
-mkGD4'4_d = mkNameG_d "base" "GHC.Generics"
-#elif MIN_VERSION_base(4,4,0)
-mkGD4'4_d = mkNameG_d "ghc-prim" "GHC.Generics"
-#else
-mkGD4'4_d = mkNameG_d gdPackageKey "Generics.Deriving.Base.Internal"
-#endif
-
-mkGD4'9_d :: String -> Name
-#if MIN_VERSION_base(4,9,0)
-mkGD4'9_d = mkNameG_d "base" "GHC.Generics"
-#else
-mkGD4'9_d = mkNameG_d gdPackageKey "Generics.Deriving.Base.Internal"
-#endif
-
-mkGD4'4_tc :: String -> Name
-#if MIN_VERSION_base(4,6,0)
-mkGD4'4_tc = mkNameG_tc "base" "GHC.Generics"
-#elif MIN_VERSION_base(4,4,0)
-mkGD4'4_tc = mkNameG_tc "ghc-prim" "GHC.Generics"
-#else
-mkGD4'4_tc = mkNameG_tc gdPackageKey "Generics.Deriving.Base.Internal"
-#endif
-
-mkGD4'9_tc :: String -> Name
-#if MIN_VERSION_base(4,9,0)
-mkGD4'9_tc = mkNameG_tc "base" "GHC.Generics"
-#else
-mkGD4'9_tc = mkNameG_tc gdPackageKey "Generics.Deriving.Base.Internal"
-#endif
-
-mkGD4'4_v :: String -> Name
-#if MIN_VERSION_base(4,6,0)
-mkGD4'4_v = mkNameG_v "base" "GHC.Generics"
-#elif MIN_VERSION_base(4,4,0)
-mkGD4'4_v = mkNameG_v "ghc-prim" "GHC.Generics"
-#else
-mkGD4'4_v = mkNameG_v gdPackageKey "Generics.Deriving.Base.Internal"
-#endif
-
-mkGD4'9_v :: String -> Name
-#if MIN_VERSION_base(4,9,0)
-mkGD4'9_v = mkNameG_v "base" "GHC.Generics"
-#else
-mkGD4'9_v = mkNameG_v gdPackageKey "Generics.Deriving.Base.Internal"
-#endif
-
-mkBaseName_d :: String -> String -> Name
-mkBaseName_d = mkNameG_d "base"
-
-mkGHCPrimName_d :: String -> String -> Name
-mkGHCPrimName_d = mkNameG_d "ghc-prim"
-
-mkGHCPrimName_tc :: String -> String -> Name
-mkGHCPrimName_tc = mkNameG_tc "ghc-prim"
-
-mkGHCPrimName_v :: String -> String -> Name
-mkGHCPrimName_v = mkNameG_v "ghc-prim"
-
 comp1DataName :: Name
-comp1DataName = mkGD4'4_d "Comp1"
+comp1DataName = 'Comp1
 
 infixDataName :: Name
-infixDataName = mkGD4'4_d "Infix"
+infixDataName = 'GD.Infix
 
 k1DataName :: Name
-k1DataName = mkGD4'4_d "K1"
+k1DataName = 'K1
 
 l1DataName :: Name
-l1DataName = mkGD4'4_d "L1"
+l1DataName = 'L1
 
 leftAssociativeDataName :: Name
-leftAssociativeDataName = mkGD4'4_d "LeftAssociative"
+leftAssociativeDataName = 'LeftAssociative
 
 m1DataName :: Name
-m1DataName = mkGD4'4_d "M1"
+m1DataName = 'M1
 
 notAssociativeDataName :: Name
-notAssociativeDataName = mkGD4'4_d "NotAssociative"
+notAssociativeDataName = 'NotAssociative
 
 par1DataName :: Name
-par1DataName = mkGD4'4_d "Par1"
+par1DataName = 'Par1
 
 prefixDataName :: Name
-prefixDataName = mkGD4'4_d "Prefix"
+prefixDataName = 'Prefix
 
 productDataName :: Name
-productDataName = mkGD4'4_d ":*:"
+productDataName = '(:*:)
 
 r1DataName :: Name
-r1DataName = mkGD4'4_d "R1"
+r1DataName = 'R1
 
 rec1DataName :: Name
-rec1DataName = mkGD4'4_d "Rec1"
+rec1DataName = 'Rec1
 
 rightAssociativeDataName :: Name
-rightAssociativeDataName = mkGD4'4_d "RightAssociative"
+rightAssociativeDataName = 'RightAssociative
 
 u1DataName :: Name
-u1DataName = mkGD4'4_d "U1"
+u1DataName = 'U1
 
 uAddrDataName :: Name
-uAddrDataName = mkGD4'9_d "UAddr"
+uAddrDataName = 'UAddr
 
 uCharDataName :: Name
-uCharDataName = mkGD4'9_d "UChar"
+uCharDataName = 'UChar
 
 uDoubleDataName :: Name
-uDoubleDataName = mkGD4'9_d "UDouble"
+uDoubleDataName = 'UDouble
 
 uFloatDataName :: Name
-uFloatDataName = mkGD4'9_d "UFloat"
+uFloatDataName = 'UFloat
 
 uIntDataName :: Name
-uIntDataName = mkGD4'9_d "UInt"
+uIntDataName = 'UInt
 
 uWordDataName :: Name
-uWordDataName = mkGD4'9_d "UWord"
+uWordDataName = 'UWord
 
 c1TypeName :: Name
-c1TypeName = mkGD4'4_tc "C1"
+c1TypeName = ''C1
 
 composeTypeName :: Name
-composeTypeName = mkGD4'4_tc ":.:"
+composeTypeName = ''(:.:)
 
 constructorTypeName :: Name
-constructorTypeName = mkGD4'4_tc "Constructor"
+constructorTypeName = ''Constructor
 
 d1TypeName :: Name
-d1TypeName = mkGD4'4_tc "D1"
+d1TypeName = ''D1
 
 genericTypeName :: Name
-genericTypeName = mkGD4'4_tc "Generic"
+genericTypeName = ''Generic
 
 generic1TypeName :: Name
-generic1TypeName = mkGD4'4_tc "Generic1"
+generic1TypeName = ''Generic1
 
 datatypeTypeName :: Name
-datatypeTypeName = mkGD4'4_tc "Datatype"
-
-noSelectorTypeName :: Name
-noSelectorTypeName = mkGD4'4_tc "NoSelector"
+datatypeTypeName = ''Datatype
 
 par1TypeName :: Name
-par1TypeName = mkGD4'4_tc "Par1"
+par1TypeName = ''Par1
 
 productTypeName :: Name
-productTypeName = mkGD4'4_tc ":*:"
+productTypeName = ''(:*:)
 
 rec0TypeName :: Name
-rec0TypeName = mkGD4'4_tc "Rec0"
+rec0TypeName = ''Rec0
 
 rec1TypeName :: Name
-rec1TypeName = mkGD4'4_tc "Rec1"
+rec1TypeName = ''Rec1
 
 repTypeName :: Name
-repTypeName = mkGD4'4_tc "Rep"
+repTypeName = ''Rep
 
 rep1TypeName :: Name
-rep1TypeName = mkGD4'4_tc "Rep1"
+rep1TypeName = ''Rep1
 
 s1TypeName :: Name
-s1TypeName = mkGD4'4_tc "S1"
+s1TypeName = ''S1
 
 selectorTypeName :: Name
-selectorTypeName = mkGD4'4_tc "Selector"
+selectorTypeName = ''Selector
 
 sumTypeName :: Name
-sumTypeName = mkGD4'4_tc ":+:"
+sumTypeName = ''(:+:)
 
 u1TypeName :: Name
-u1TypeName = mkGD4'4_tc "U1"
+u1TypeName = ''U1
 
 uAddrTypeName :: Name
-uAddrTypeName = mkGD4'9_tc "UAddr"
+uAddrTypeName = ''UAddr
 
 uCharTypeName :: Name
-uCharTypeName = mkGD4'9_tc "UChar"
+uCharTypeName = ''UChar
 
 uDoubleTypeName :: Name
-uDoubleTypeName = mkGD4'9_tc "UDouble"
+uDoubleTypeName = ''UDouble
 
 uFloatTypeName :: Name
-uFloatTypeName = mkGD4'9_tc "UFloat"
+uFloatTypeName = ''UFloat
 
 uIntTypeName :: Name
-uIntTypeName = mkGD4'9_tc "UInt"
+uIntTypeName = ''UInt
 
 uWordTypeName :: Name
-uWordTypeName = mkGD4'9_tc "UWord"
+uWordTypeName = ''UWord
 
 v1TypeName :: Name
-v1TypeName = mkGD4'4_tc "V1"
+v1TypeName = ''V1
 
 conFixityValName :: Name
-conFixityValName = mkGD4'4_v "conFixity"
+conFixityValName = 'conFixity
 
 conIsRecordValName :: Name
-conIsRecordValName = mkGD4'4_v "conIsRecord"
+conIsRecordValName = 'conIsRecord
 
 conNameValName :: Name
-conNameValName = mkGD4'4_v "conName"
+conNameValName = 'GD.conName
 
 datatypeNameValName :: Name
-datatypeNameValName = mkGD4'4_v "datatypeName"
+datatypeNameValName = 'GD.datatypeName
 
 isNewtypeValName :: Name
-isNewtypeValName = mkGD4'4_v "isNewtype"
+isNewtypeValName = 'isNewtype
 
 fromValName :: Name
-fromValName = mkGD4'4_v "from"
+fromValName = 'from
 
 from1ValName :: Name
-from1ValName = mkGD4'4_v "from1"
+from1ValName = 'from1
 
 moduleNameValName :: Name
-moduleNameValName = mkGD4'4_v "moduleName"
+moduleNameValName = 'moduleName
 
 selNameValName :: Name
-selNameValName = mkGD4'4_v "selName"
+selNameValName = 'selName
 
 seqValName :: Name
-seqValName = mkGHCPrimName_v "GHC.Prim" "seq"
+seqValName = 'seq
 
 toValName :: Name
-toValName = mkGD4'4_v "to"
+toValName = 'to
 
 to1ValName :: Name
-to1ValName = mkGD4'4_v "to1"
+to1ValName = 'to1
 
 uAddrHashValName :: Name
-uAddrHashValName = mkGD4'9_v "uAddr#"
+uAddrHashValName = 'uAddr#
 
 uCharHashValName :: Name
-uCharHashValName = mkGD4'9_v "uChar#"
+uCharHashValName = 'uChar#
 
 uDoubleHashValName :: Name
-uDoubleHashValName = mkGD4'9_v "uDouble#"
+uDoubleHashValName = 'uDouble#
 
 uFloatHashValName :: Name
-uFloatHashValName = mkGD4'9_v "uFloat#"
+uFloatHashValName = 'uFloat#
 
 uIntHashValName :: Name
-uIntHashValName = mkGD4'9_v "uInt#"
+uIntHashValName = 'uInt#
 
 uWordHashValName :: Name
-uWordHashValName = mkGD4'9_v "uWord#"
+uWordHashValName = 'uWord#
 
 unComp1ValName :: Name
-unComp1ValName = mkGD4'4_v "unComp1"
+unComp1ValName = 'unComp1
 
 unK1ValName :: Name
-unK1ValName = mkGD4'4_v "unK1"
+unK1ValName = 'unK1
 
 unPar1ValName :: Name
-unPar1ValName = mkGD4'4_v "unPar1"
+unPar1ValName = 'unPar1
 
 unRec1ValName :: Name
-unRec1ValName = mkGD4'4_v "unRec1"
+unRec1ValName = 'unRec1
 
 trueDataName, falseDataName :: Name
-#if MIN_VERSION_base(4,4,0)
-trueDataName  = mkGHCPrimName_d "GHC.Types" "True"
-falseDataName = mkGHCPrimName_d "GHC.Types" "False"
-#else
-trueDataName  = mkGHCPrimName_d "GHC.Bool"  "True"
-falseDataName = mkGHCPrimName_d "GHC.Bool"  "False"
-#endif
+trueDataName  = 'True
+falseDataName = 'False
 
 nothingDataName, justDataName :: Name
-#if MIN_VERSION_base(4,12,0)
-nothingDataName = mkBaseName_d "GHC.Maybe"  "Nothing"
-justDataName    = mkBaseName_d "GHC.Maybe"  "Just"
-#elif MIN_VERSION_base(4,8,0)
-nothingDataName = mkBaseName_d "GHC.Base"   "Nothing"
-justDataName    = mkBaseName_d "GHC.Base"   "Just"
-#else
-nothingDataName = mkBaseName_d "Data.Maybe" "Nothing"
-justDataName    = mkBaseName_d "Data.Maybe" "Just"
-#endif
-
-mkGHCPrim_tc :: String -> Name
-mkGHCPrim_tc = mkNameG_tc "ghc-prim" "GHC.Prim"
+nothingDataName = 'Nothing
+justDataName    = 'Just
 
 addrHashTypeName :: Name
-addrHashTypeName = mkGHCPrim_tc "Addr#"
+addrHashTypeName = ''Addr#
 
 charHashTypeName :: Name
-charHashTypeName = mkGHCPrim_tc "Char#"
+charHashTypeName = ''Char#
 
 doubleHashTypeName :: Name
-doubleHashTypeName = mkGHCPrim_tc "Double#"
+doubleHashTypeName = ''Double#
 
 floatHashTypeName :: Name
-floatHashTypeName = mkGHCPrim_tc "Float#"
+floatHashTypeName = ''Float#
 
 intHashTypeName :: Name
-intHashTypeName = mkGHCPrim_tc "Int#"
+intHashTypeName = ''Int#
 
 wordHashTypeName :: Name
-wordHashTypeName = mkGHCPrim_tc "Word#"
+wordHashTypeName = ''Word#
 
 composeValName :: Name
-composeValName = mkNameG_v "base" "GHC.Base" "."
+composeValName = '(.)
 
 errorValName :: Name
-errorValName = mkNameG_v "base" "GHC.Err" "error"
+errorValName = 'error
 
 fmapValName :: Name
-fmapValName = mkNameG_v "base" "GHC.Base" "fmap"
+fmapValName = 'fmap
 
 undefinedValName :: Name
-undefinedValName = mkNameG_v "base" "GHC.Err" "undefined"
+undefinedValName = 'undefined
 
 decidedLazyDataName :: Name
-decidedLazyDataName = mkGD4'9_d "DecidedLazy"
+decidedLazyDataName = 'GD.DecidedLazy
 
 decidedStrictDataName :: Name
-decidedStrictDataName = mkGD4'9_d "DecidedStrict"
+decidedStrictDataName = 'GD.DecidedStrict
 
 decidedUnpackDataName :: Name
-decidedUnpackDataName = mkGD4'9_d "DecidedUnpack"
+decidedUnpackDataName = 'GD.DecidedUnpack
 
 infixIDataName :: Name
-infixIDataName = mkGD4'9_d "InfixI"
+infixIDataName = 'InfixI
 
 metaConsDataName :: Name
-metaConsDataName = mkGD4'9_d "MetaCons"
+metaConsDataName = 'MetaCons
 
 metaDataDataName :: Name
-metaDataDataName = mkGD4'9_d "MetaData"
-
-metaNoSelDataName :: Name
-metaNoSelDataName = mkGD4'9_d "MetaNoSel"
+metaDataDataName = 'MetaData
 
 metaSelDataName :: Name
-metaSelDataName = mkGD4'9_d "MetaSel"
+metaSelDataName = 'MetaSel
 
 noSourceStrictnessDataName :: Name
-noSourceStrictnessDataName = mkGD4'9_d "NoSourceStrictness"
+noSourceStrictnessDataName = 'GD.NoSourceStrictness
 
 noSourceUnpackednessDataName :: Name
-noSourceUnpackednessDataName = mkGD4'9_d "NoSourceUnpackedness"
+noSourceUnpackednessDataName = 'GD.NoSourceUnpackedness
 
 prefixIDataName :: Name
-prefixIDataName = mkGD4'9_d "PrefixI"
+prefixIDataName = 'PrefixI
 
 sourceLazyDataName :: Name
-sourceLazyDataName = mkGD4'9_d "SourceLazy"
+sourceLazyDataName = 'GD.SourceLazy
 
 sourceNoUnpackDataName :: Name
-sourceNoUnpackDataName = mkGD4'9_d "SourceNoUnpack"
+sourceNoUnpackDataName = 'GD.SourceNoUnpack
 
 sourceStrictDataName :: Name
-sourceStrictDataName = mkGD4'9_d "SourceStrict"
+sourceStrictDataName = 'GD.SourceStrict
 
 sourceUnpackDataName :: Name
-sourceUnpackDataName = mkGD4'9_d "SourceUnpack"
+sourceUnpackDataName = 'GD.SourceUnpack
 
 packageNameValName :: Name
-packageNameValName = mkGD4'4_v "packageName"
+packageNameValName = 'packageName
